import { Collection } from "https://deno.land/x/mongo@v0.31.2/mod.ts";
import { UUID } from "https://deno.land/x/web_bson@v0.3.0/mod.js";

import { auth, body, jwt, page, rest, schema } from "../server/decorators.ts";
import { JWTPayload } from "../utils/JWT.ts";
import AppRouter from "./AppRouter.ts";
import HttpError from "../server/HttpError.ts";
import User from "../models/User.ts";
import Image, { Visibility } from "../models/Image.ts";
import { Page } from "../server/decorators/page.ts";

export default class ImageRouter extends AppRouter {
    public static serialize(image: Image) {
        return {
            author: image.author,
            created: image.created.getTime() / 1000,
            name: image.name,
            visibility: image.visibility,
            id: image._id,
        };
    }

    @rest('GET', '/')
    async get(@schema('uuid') id: UUID, @jwt(v => v.salt, false) @auth() jwt?: JWTPayload) {
        const image = await this.images.findOne({ _id: new UUID(id) });

        if (
            !image ||
            image.visibility === Visibility.Private && image.author !== jwt?.name
        ) throw new HttpError("Image doesn't exist.");

        return ImageRouter.serialize(image);
    }

    @rest('GET', '/mine')
    async mine(@page() page: Page, @jwt(v => v.salt, true) @auth() jwt: JWTPayload) {
        const user = await this.users.findOne({ username: jwt.name });
        if (!user) throw new HttpError("You don't exist.");
        const res = await page.apply(this.images.find({ _id: { $in: user.images } })).toArray();
        return res.map(v => ImageRouter.serialize(v));
    }

    public constructor(private salt: string, private images: Collection<Image>, private users: Collection<User>) {
        super();
        users.createIndexes({ indexes: [ { key: { username: 1 }, name: 'Username Index' } ] });
    }
}